% -------------------------------------------------------------------------
% Script to calibrate the Kinect Camera to the Fanuc Robot
% Moves robot to 64 locations and locates X in Kinect image and then computes
% the transformation matrix between the Kinect and Robot. Creates TransMatrix.mat
% which contains transformation matrix T. Stores image data for post processing in 
% calibrationInput.mat
% -------------------------------------------------------------------------
% Launch the teaching pendant program LV_COM_FRM before launching this script.
% See: http://classes.engineering.wustl.edu/ese497/index.php/Fanuc_How_Tos,_FAQ,_etc.#FAQ
% for detailed instructions.
% -------------------------------------------------------------------------
% See comments at top of Matlab2Fanuc for details of the robot communication
% See comments at the top of Kinect2SharedMem2Matlab.m for details of the
% Kinect camera communication.
% -------------------------------------------------------------------------
clear all; close all;
collectData = 1 ;

host='localhost';
%host='Fanucomp.seasad.wustl.edu';

% Launch LabVIEW program that receives TCP/IP commands and communicates
% with the Robot
!start /min "" LabVIEW2Fanuc\builds\MatlabLabVIEWFanuc\MatlabLabVIEWFanuc.exe
pause(2) ; % Make sure this is open so we don't have communication errors

h1 = figure(11) ;
h2 = figure(12) ;
h3 = figure(13) ;
h4 = figure(14) ;
h5 = figure(15) ;

% Initialize Kinect and Shared Memory
if ( collectData )
    kinectCam = Kinect() ;
    [ cross, templateX ] = CreateTemplates() ;
    rangeXYZ=[[375,525];[-150,150];[-250,-150]] ;
    numSteps = 4 ;
    rangeInc = (rangeXYZ(:,2) - rangeXYZ(:,1))/(numSteps-1) ;
    searchWinX = [130,510] ;
    searchWinY = [100,410] ;
else
    if ( ~exist('firstRun') )
        load ('calibrationInput.mat','numSteps','rangeXYZ','rangeInc','rgb','depth','searchWinX','searchWinY','cross','templateX');
        firstRun = 1 ;
        collectData = 0 ;
    end
end
frame = 1 ;
for i = 1:numSteps
    x = rangeXYZ(1,1)+(i-1)*rangeInc(1) ;
    for j = 1:numSteps
        y = rangeXYZ(2,1)+(j-1)*rangeInc(2) ;
        for k = 1:numSteps
            z = rangeXYZ(3,1)+(k-1)*rangeInc(3) ;
            fanuc(frame,1) = x ;
            fanuc(frame,2) = y ;
            fanuc(frame,3) = z ;
            if ( collectData )
                [data_received]=exchangeData(host,Double2CSV([x y z -90 -90 -90 0 0]));
               
                pause(1) ;
                [rgb(:,:,:,frame),depth(:,:,frame)] = kinectCam.GetImage() ;
            end
            figure(h1) ; h1imagesc = imagesc(depth(:,:,frame)) ;
            caxis([350 1000]) ;
            figure(h2) ; h2image = image(rgb(:,:,:,frame)) ;

            depthWin = depth(searchWinY(1):searchWinY(2),searchWinX(1):searchWinX(2),frame) ;
            rgbWin = rgb(searchWinY(1):searchWinY(2),searchWinX(1):searchWinX(2),:,frame) ;

            template = templateX ;
            c1 = normxcorr2(template,rgb2gray(rgbWin)) ;
            c = c1((size(template,1)-1)/2+1:(size(template,1)-1)/2+size(rgbWin,1),(size(template,2)-1)/2+1:(size(template,2)-1)/2+size(rgbWin,2)) ;
            figure(h3) ; imagesc(c) ;
            highCCIndices = find(c > 0.5) ;

            [maxCC, indxMaxCC] = max(c(highCCIndices)) ;
            target(frame,1) = floor((highCCIndices(indxMaxCC)-1)/size(rgbWin,1)) + 1 + searchWinX(1) -1 ;
            target(frame,2) = mod((highCCIndices(indxMaxCC)-1),size(rgbWin,1))+ 1 + searchWinY(1) -1 ;
            medianDepth = DepthFilter(depth(target(frame,2)-20:target(frame,2)+20, ...
            target(frame,1)-20:target(frame,1)+20,frame),3,10) ;
            target(frame,3) = medianDepth(21,21) ;
            figure(h2) ; hold on ; plot(target(frame,1),target(frame,2),'rx') ; hold off ;
            drawnow
            frame = frame + 1 ;
        end
    end
end
% See FindBestFOVs.m to measure best FOV for this Kinect
cart_mm = proj2cart(target,size(depth,2),size(depth,1),63.1,49.6) ;
% figure ;plot3(cart_mm(:,1),cart_mm(:,2),cart_mm(:,3),'bx') ; xlabel('x') ; ylabel('y') ; zlabel('z') ; grid on ;
path(path,'./absor') ;
[regParams,Bfit,ErrorStats]=absor(cart_mm',fanuc','doScale',true);
T=regParams.M ;
% Compute error - appending a one into the bottom row of cart_mm
F = T*([cart_mm'; ones(1,size(cart_mm,1))]) ;
F = F(1:3,:) ;
figure(h4) ;
plot3(fanuc(:,1),fanuc(:,2),fanuc(:,3),'bx') ; xlabel('x') ; ylabel('y') ; zlabel('z') ; grid on ;
hold on ;
plot3(F(1,:),F(2,:),F(3,:),'go') ;
hold off ;
legend('Ideal','Transformed') ;
error = F - fanuc' ;
figure(h5) ; plot(sqrt(diag(error'*error)),'rx')
% Shutdown Kinect and Shared Memory
if ( collectData )
    kinectCam.ShutDown() ;
    % Kill (ungracefully) LabVIEW communication program
    !taskkill /IM MatlabLabVIEWFanuc.exe
    save calibrationInput.mat
    %Save just the Transform Matrix
    %This is what should be loaded and used in future programs
    save('TransMatrix.mat','T')

end

